%  alarmResult=challenge(recordName,alarm_type)
%
% Sample entry for the 2015 PhysioNet/CinC Challenge.
%
% Inputs:
%   recordName
%       String specifying the record name to process
%   alarmType
%       String specifying the alarm type. Alarm types are:
%             Asystole, Bradycardia, Tachycardia,
%             Ventricular_Tachycardia, Ventricular_Flutter_Fib
%
%
% Outputs:
%   alarmResult
%       Integer value where 0 = false alarm and 1 is a true
%       alarm. 
%
%
% To run your entry on the entire training set in a format that is
% compatible with PhysioNet's scoring enviroment, run the script
% generateValidationSet.m
%
% Dependencies:
%
%       1) This function does not requires that you have the WFDB
%       App Toolbox installed. 
%       A matlab function 'rdmat' can read the data instead of using WFDB
%       Toolbox.
%
%       2) The CHALLENGE function requires that you have downloaded the challenge
%       data 'set-p' in a subdirectory of the current directory. The subdirectory
%       should be called '/challenge/2015/set-p/' . The 'set-p' dataset can
%       be downloaded from PhysioNet at:
%           http://physionet.org/physiobank/database/challenge/2015/
%          
%         This dataset is used by the generateValidationSet.m script to
%         create the annotations on your traing set that will be used to 
%         verify that your entry works properly in the PhysioNet testing 
%         environment. 
%
% Version 0.5
%
%
% Written by Qiao Li, November 10, 2014.
% Last Modified: Ikaro Silva February 11, 2015
%
%
%
% %Example using training data- 
% alarmResult=challenge('./challenge/set-p/100','Asystole')
%

% Name of file containing answers
answers = 'hello.txt';

%Get all ECG, blood pressure and photoplethysmogram signals
[~,signal,Fs,siginfo]=rdmat(recordName);
alarmResult=1;

% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % %%%%%%%%%%% Nadi 18/3/2015
% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

description=squeeze(struct2cell(siginfo));
description=description(4,:);

% Resample signal to 125Hz
Fs=Fs(1);
if Fs~=125
    signal=resample(signal,125,Fs);
    Fs=125;
end

% Adding ECG signals
ecg_signal = signal(:,1);
ecg_signal = medfilt1(ecg_signal);
Rpeaks     = QRSdet_Hilbert(ecg_signal',Fs);
% Rpeaks5=QRSdet_Hilbert(signal(:,2)',Fs);

% ff=1/Fs;
% peaks = PeakDetection(signal(:,1),ff);
% Rpeaks=find((peaks)>0.5*max(peaks));
% 
% peaks5 = PeakDetection(signal(:,2),ff);
% Rpeaks5=find((peaks5)>0.5*max(peaks5));


%%Users can access the raw samples of the record by running the following
%command if WFDB Toolbox installed:
%[tm,signal]=rdsamp(recordName);
%
%%For more information please see the help in RDSAMP

%Run WABP on the record, which by default will analyze the first ABP, ART, or BP signal
N=[];
N0=[];
abp_ind=get_index(description,'ABP');
ann_abp=[];
features=[];
BEATQ=[];
R=[];
if(~isempty(abp_ind))
   ann_abp=wabp(signal(:,abp_ind),0,1);
   % Analyze the signal quality index of ABP using jSQI
   if length(ann_abp)>=3 % at least 3 abp beats detected
        [features] = abpfeature(signal(:,abp_ind),ann_abp);
        [BEATQ R] = jSQI(features, ann_abp, signal(:,abp_ind));
   end
end

%Run WABP on the record of 'PLETH' to analyze photoplethysmogram signal
ppg_ind=get_index(description,'PLETH');
ann_ppg=[];
if (~isempty(ppg_ind))
    y=quantile(signal(:,ppg_ind),[0.05,0.5,0.95]);
    ann_ppg=wabp(signal(:,ppg_ind),0,(y(3)-y(1))/120);
    % Analyze the signal quality index of PPG 
    if ~isempty(ann_ppg)
        [psqi]=ppgSQI(signal(:,ppg_ind),ann_ppg);
    end
end



%Make decisions

% set valid data segment for decision making, 16s before the alarm
N_d=Fs*5*60; % alarm position
N0_d=N_d-Fs*16+1; % 16s before the alarm

% select the beats in the segment
n_abp_beats=intersect(find(ann_abp>=N0_d),find(ann_abp<=N_d));
n_ppg_beats=intersect(find(ann_ppg>=N0_d),find(ann_ppg<=N_d));
%Added ECG II
n_ECG2_beats=intersect(find(Rpeaks>=N0_d),find(Rpeaks<=N_d));
%Added ECG V
% n_ECG5_beats=intersect(find(Rpeaks5>=N0_d),find(Rpeaks5<=N_d));

hr_max_abp=NaN;
max_rr_abp=NaN;
max_rr_abp=NaN;
hr_max_ppg=NaN;
max_rr_ppg=NaN;
hr_max_ECG2=NaN;
max_rr_ECG2=NaN;
% hr_max_ECG5=NaN;
% max_rr_ECG5=NaN;

% calculate the heart rate
if length(n_abp_beats)>=2
    hr_max_abp=60*Fs/min(diff(ann_abp(n_abp_beats)));
    max_rr_abp=max(diff(ann_abp(n_abp_beats)))/Fs;
end
if length(n_ppg_beats)>=2
    hr_max_ppg=60*Fs/min(diff(ann_ppg(n_ppg_beats)));
    max_rr_ppg=max(diff(ann_ppg(n_ppg_beats)))/Fs;
end
%Added ECG II
if length(n_ECG2_beats)>=2
    hr_max_ECG2=60*Fs/min(diff(Rpeaks(n_ECG2_beats)));
    max_rr_ECG2=max(diff(Rpeaks(n_ECG2_beats)))/Fs;
end

%rough P-wave detection for VTach
l_ECG2_window=Fs*0.2; %Search in window of 200 ms
l_ecg_vt_beats = 0;
diff_th = 0.01;
zero_th = 0.001;
for i=1:length(n_ECG2_beats)
    ecg_sig_dt = diff(ecg_signal(Rpeaks(n_ECG2_beats(i))-l_ECG2_window:Rpeaks(n_ECG2_beats(i))));
    ecg_sig_dt2 = diff(ecg_sig_dt);
    ecg_p_idx = find(ecg_sig_dt==0);
    ecg_p_idx(ecg_p_idx==1)=[];
    cnt = 1;
    
    if ~isempty(ecg_p_idx)
        for x=1:length(ecg_p_idx)
            idx = ecg_p_idx(x);
            if (ecg_sig_dt2(idx-1) < -zero_th && ecg_sig_dt2(idx+1)>zero_th) %% p-wave detected
                cnt = 0;
            end
        end
    end
    l_ecg_vt_beats = l_ecg_vt_beats + cnt;
end
%Added ECG V
% if length(n_ECG5_beats)>=2
%     hr_max_ECG5=60*Fs/min(diff(Rpeaks5(n_ECG5_beats)));
%     max_rr_ECG5=max(diff(Rpeaks5(n_ECG5_beats)))/Fs;
% end
    
% calculate low heart rate of 5 consecutive beats for Bradycardia
low_hr_abp=NaN;
low_hr_ppg=NaN;
low_hr_ECG2=NaN;
% low_hr_ECG5=NaN;
if length(n_abp_beats>=5)
    for i=1:length(n_abp_beats)-4
        low_hr_abp(i)=60*Fs/((ann_abp(n_abp_beats(i+4))-ann_abp(n_abp_beats(i)))/4);
    end
end
low_hr_abp=min(low_hr_abp);
if length(n_ppg_beats>=5)
    for i=1:length(n_ppg_beats)-4
        low_hr_ppg(i)=60*Fs/((ann_ppg(n_ppg_beats(i+4))-ann_ppg(n_ppg_beats(i)))/4);
    end
end
low_hr_ppg=min(low_hr_ppg);
% Adding ECG II        
if length(n_ECG2_beats>=5)
    for i=1:length(n_ECG2_beats)-4
        low_hr_ECG2(i)=60*Fs/((Rpeaks(n_ECG2_beats(i+4))-Rpeaks(n_ECG2_beats(i)))/4);
    end
end
low_hr_ECG2=min(low_hr_ECG2);
% Adding ECG V        
% if length(n_ECG5_beats>=5)
%     for i=1:length(n_ECG5_beats)-4
%         low_hr_ECG5(i)=60*Fs/((Rpeaks5(n_ECG5_beats(i+4))-Rpeaks5(n_ECG5_beats(i)))/4);
%     end
% end
% low_hr_ECG5=min(low_hr_ECG5);


% calculate high heart reate of 17 consecutive beats for Tachycardia
high_hr_abp=NaN;
high_hr_ppg=NaN;
high_hr_ECG2=NaN;
% high_hr_ECG5=NaN;
if length(n_abp_beats>=17)
    for i=1:length(n_abp_beats)-16
        high_hr_abp(i)=60*Fs/((ann_abp(n_abp_beats(i+16))-ann_abp(n_abp_beats(i)))/16);
    end
end
high_hr_abp=max(high_hr_abp);
if length(n_ppg_beats>=17)
    for i=1:length(n_ppg_beats)-16
        high_hr_ppg(i)=60*Fs/((ann_ppg(n_ppg_beats(i+16))-ann_ppg(n_ppg_beats(i)))/16);
    end
end
high_hr_ppg=max(high_hr_ppg);
% Adding ECG II
if length(n_ECG2_beats>=17)
    for i=1:length(n_ECG2_beats)-16
        high_hr_ECG2(i)=60*Fs/((Rpeaks(n_ECG2_beats(i+16))-Rpeaks(n_ECG2_beats(i)))/16);
    end
end
high_hr_ECG2=max(high_hr_ECG2);
% % Adding ECG V
% if length(n_ECG5_beats>=17)
%     for i=1:length(n_ECG5_beats)-16
%         high_hr_ECG5(i)=60*Fs/((Rpeaks5(n_ECG5_beats(i+16))-Rpeaks5(n_ECG5_beats(i)))/16);
%     end
% end
% high_hr_ECG5=max(high_hr_ECG5);

% calculate the signal quality index
if ~isempty(ann_abp)
    abpsqi=1-sum(sum(BEATQ(intersect(n_abp_beats,1:length(BEATQ)),:)))/numel(BEATQ(intersect(n_abp_beats,1:length(BEATQ)),:));
else
    abpsqi=0;
end
if ~isempty(ann_ppg)
    ppgsqi=mean(psqi(intersect(n_ppg_beats,1:length(psqi))));
else
    ppgsqi=0;
end

% Adding ECG II & V
% Just adding a condition of max_rr_ECG2<6: Meaning that if the RR interval
% has been greater than 6 seconds, assume the ECG signal as low quality and
% do not consider it in arrhythmia detection.

% SQI threshold
sqi_th = 0.95;

% Alarm threshold (seconds)
ASY_th = 4;
BRA_th = 40;
TAC_th = 140;
VTA_th = 100;
VFB_th = 250;
VTA_beats_mx = 5;
tolerance = 10; % tolerance = 5 bmp
switch alarm_type
    case 'Asystole'
        % if the signal quality is good enough and the maximum RR interval
        % is less than the Asystole threshold, set the alarm as 'F'
        if (abpsqi>=sqi_th && max_rr_abp<ASY_th) || (ppgsqi>=sqi_th && max_rr_ppg<ASY_th) || (max_rr_ECG2<6 && max_rr_ECG2<ASY_th) %|| (max_rr_ECG5<6 && max_rr_ECG5<ASY_th)
            alarmResult=0;
        end
    case 'Bradycardia'
        % if the signal quality is good enough and the low heart rate
        % is greater than the Bradycardia threshold, set the alarm as 'F'
        if (abpsqi>=sqi_th && low_hr_abp-tolerance>BRA_th) || (ppgsqi>=sqi_th && low_hr_ppg-tolerance>BRA_th) || (max_rr_ECG2<6 && low_hr_ECG2-tolerance>BRA_th) %|| (max_rr_ECG5<6 && low_hr_ECG5-tolerance>BRA_th)
            alarmResult=0;
        end
    case 'Tachycardia'
        % if the signal quality is good enough and the high heart rate
        % is less than the Tachycardia threshold, set the alarm as 'F'
        if (abpsqi>=sqi_th && high_hr_abp+tolerance<TAC_th) || (ppgsqi>=sqi_th && high_hr_ppg+tolerance<TAC_th) || (max_rr_ECG2<6 && high_hr_ECG2+tolerance<TAC_th) %|| (max_rr_ECG5<6 && high_hr_ECG5+tolerance<TAC_th)
            alarmResult=0;
        end
    case 'Ventricular_Tachycardia'
        % suppress false alarm using hr_max & sqi
        if (abpsqi>=sqi_th && hr_max_abp+tolerance<VTA_th) || (ppgsqi>=sqi_th && hr_max_ppg+tolerance<VTA_th) || (max_rr_ECG2<6 && hr_max_ECG2+tolerance<VTA_th) || (l_ecg_vt_beats < VTA_beats_mx) %|| (max_rr_ECG5<6 && hr_max_ECG5+tolerance<VTA_th)
            alarmResult=0;
        end
    case 'Ventricular_Flutter_Fib'
        % suppress false alarm using hr_max & sqi
        if (abpsqi>=sqi_th && hr_max_abp+tolerance<VFB_th) || (ppgsqi>=sqi_th && hr_max_ppg+tolerance<VFB_th) || (max_rr_ECG2<6 && hr_max_ECG2+tolerance<VFB_th) %|| (max_rr_ECG5<6 && hr_max_ECG5+tolerance<VFB_th)
            alarmResult=0;
        end
    otherwise
        error(['Unknown alarm type: ' alarm_type])
end

% Write result to answers.txt
fid = fopen(answers, 'a');
if (fid == -1)
    error('Could not open answer file');
end

fprintf(fid, '%s,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\r\n', alarm_type, alarmResult, gold, abpsqi, low_hr_abp, hr_max_abp, max_rr_abp, high_hr_ppg, max_rr_ppg, hr_max_ppg, max_rr_ECG2, high_hr_ECG2, hr_max_ECG2);
fclose(fid);

end